@@ -1,4 +1,4 @@
-# Copyright (C) 1998,1999,2000 by the Free Software Foundation, Inc.
+# Copyright (C) 1998,1999,2000,2001 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -22,6 +22,7 @@ import os
 import cgi
 import string
 import types
+import rfc822
 
 from Mailman import Utils
 from Mailman import MailList
@@ -835,10 +836,13 @@ def ChangeOptions(mlist, category, cgi_info, document):
     #
     # mass subscription processing for members category
     #
+    def clean_names(name):
+        return rfc822.unquote(string.strip(name))
+
     if cgi_info.has_key('subscribees'):
 	name_text = cgi_info['subscribees'].value
         name_text = string.replace(name_text, '\r', '')
-	names = filter(None, map(string.strip, string.split(name_text, '\n')))
+	names = filter(None, map(clean_names, string.split(name_text, '\n')))
         send_welcome_msg = string.atoi(
             cgi_info["send_welcome_msg_to_this_batch"].value)
         digest = 0
