@@ -1,4 +1,4 @@
-# Copyright (C) 1998 by the Free Software Foundation, Inc.
+# Copyright (C) 1998,1999,2000 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -32,12 +32,25 @@ from types import StringType
 # XXX: obsolete, should use re module
 import regsub
 import random
-import mm_cfg
-import Errors
+import urlparse
+
+from Mailman import mm_cfg
+from Mailman import Errors
 ##try:
 ##    import md5
 ##except ImportError:
 ##    md5 = None
+from Mailman import Crypt
+
+
+
+def list_exists(listname):
+    """Return true iff list `listname' exists."""
+    # It is possible that config.db got removed erroneously, in which case we
+    # can fall back to config.db.last
+    dbfile = os.path.join(mm_cfg.LIST_DATA_DIR, listname, 'config.db')
+    lastfile = dbfile + '.last'
+    return os.path.exists(dbfile) or os.path.exists(lastfile)
 
 
 
@@ -45,14 +58,11 @@ def list_names():
     """Return the names of all lists in default list directory."""
     got = []
     for fn in os.listdir(mm_cfg.LIST_DATA_DIR):
-	if not (
-	    os.path.exists(
-		os.path.join(os.path.join(mm_cfg.LIST_DATA_DIR, fn),
-			     'config.db'))):
-	    continue
-	got.append(fn)
+        if list_exists(fn):
+            got.append(fn)
     return got
 
+
 
 # a much more naive implementation than say, Emacs's fill-paragraph!
 def wrap(text, column=70):
@@ -131,210 +141,6 @@ def wrap(text, column=70):
     return wrapped[:-2]
     
 
-
-def SendTextToUser(subject, text, recipient, sender, add_headers=[]):
-    import Message
-    msg = Message.OutgoingMessage()
-    msg.SetSender(sender)
-    msg.SetHeader('Subject', subject, 1)
-    msg.SetBody(QuotePeriods(text))
-    DeliverToUser(msg, recipient, add_headers=add_headers)
-
-
-
-_children = []
-
-def DeliverToUser(msg, recipient, add_headers=[]):
-    """Use smtplib to deliver message.
-
-    Optional argument add_headers should be a list of headers to be added
-    to the message, e.g. for Errors-To and X-No-Archive."""
-    # HM: I don't think the deadlock comment below really applies
-    # anymore (because we now send mail with SMTP, not by invoking some
-    # binary).  Thus the forking should probably go away (eventually,
-    # i.e. when we're sure that won't break stuff).  For now, trying to
-    # approach 1.0, I've only moved the forking down a bit.
-    #
-    # We fork to ensure no deadlock.  Otherwise, even if sendmail is
-    # invoked in forking mode, if it eg detects a bad address before
-    # forking, then it will try deliver to the errorsto addr *in the
-    # foreground*.  If the errorsto happens to be the list owner for a list
-    # that is doing the send - and holding a lock - then the delivery will
-    # hang pending release of the lock - deadlock.
-    global _children
-    sender = msg.GetSender()
-    try:
-        msg.headers.remove('\n')
-    except ValueError:
-        pass
-    if not msg.getheader('to'):
-        msg.headers.append('To: %s\n' % recipient)
-    for i in add_headers:
-        if i and i[-1] != '\n':
-            i = i + '\n'
-        msg.headers.append(i)
-    text = string.join(msg.headers, '') + '\n' + QuotePeriods(msg.body)
-    import OutgoingQueue
-    queue_id = OutgoingQueue.enqueueMessage(sender, recipient, text)
-    # If the fork (or anything the child does) fails and raises an
-    # exception, e.g. due to memory exhaustion, the queue runner
-    # will pick up the queue file.
-    try:
-        pid = os.fork()
-        if pid:
-            # in the parent
-            _children.append(pid)
-        else:
-            # in the child
-            try:
-                TrySMTPDelivery(recipient,sender,text,queue_id)
-            finally:
-                os._exit(0)
-    except:
-        OutgoingQueue.deferMessage(queue_id)
-
-
-def Reap():
-    global _children
-    try:
-        pid, sts = os.waitpid(-1, os.WNOHANG)
-        try:
-            _children.remove(pid)
-        except ValueError:
-            pass
-    except os.error:
-        pass
-    return len(_children)
-
-
-
-def TrySMTPDelivery(recipient, sender, text, queue_entry):
-    import socket
-    import OutgoingQueue
-    from Logging.StampedLogger import StampedLogger
-    # We need to get a modern smtplib.  The Python 1.5.1 version does not
-    # support ehlo (ESMTP), but the Python 1.5.2 version does.  However, the
-    # interface to conn.ehlo() has changed between Python 1.5.2beta and 1.5.2
-    # final.  Also, there was a patch to smtplib.py after Python 1.5.2 final
-    # was released.  For all these reasons we import smtplib directly out of
-    # the Mailman.pythonlib package.  This contains the latest (as of
-    # 27-Apr-1999) smtplib.py file from the Python CVS tree.
-    from Mailman.pythonlib import smtplib
-    try:
-        conn = smtplib.SMTP(mm_cfg.SMTPHOST)
-        # Do the EHLO/HELO manually so we can check for DSN support
-        if conn.ehlo()[0] >= 400:
-            conn.helo()
-        # receipt opts, empty unless DSN is supported, in which case we only
-        # want notification on failures
-        ropts = []
-        if conn.has_extn('dsn'):
-            ropts.append('NOTIFY=failure')
-        # With a global listmember database, we could intercept RCPT errors
-        # here to disable/remove bad local addresses.  Currently we don't
-        # know which list(s) the address is on :(
-        refused = conn.sendmail(sender, recipient, text, rcpt_options=ropts)
-        #
-        # debugging
-        #
-##        l = StampedLogger('smtp-delivery', 'TrySMTPDelivery', immediate=1)
-##        sys.stdout = l
-##        try:
-##            print 'SMTP delivery:',
-##            if refused: print 'partial'
-##            else: print 'full'
-##            print 'queue_entry:', queue_entry
-##            print 'sender:', sender
-##            print 'recipient:', recipient
-##            if md5:
-##                texthash = md5.new(text).digest()
-##                x = []
-##                for c in texthash:
-##                    z = hex(ord(c))[2:]
-##                    if len(z) == 1: z = '0'+z
-##                    x.append(z)
-##                print 'md5 of text:', string.join(x, '')
-##            if refused:
-##                print 'refused:', refused
-##        finally:
-##            sys.stdout = sys.__stdout__
-        #
-        # end debugging
-        #
-        conn.quit()
-        defer = 0
-        failure = None
-    # Any exceptions that warrant leaving the message on the queue should be
-    # identified by their exception, below, with setting 'defer' to 1 and
-    # 'failure' to something suitable.  Without a particular exception we fall
-    # through to the blanket 'except:', which dequeues the message.
-    # 
-    except (# MTA not responding, or other socket prob
-            socket.error,
-            # Unusual HELO response, hopefully due to transient problems
-            smtplib.SMTPHeloError):
-        # Leave the message on queue
-        defer = 1
-        failure = sys.exc_info()
-        log_hint = "Maybe your MTA daemon needs restarting?"
-    except smtplib.SMTPSenderRefused:
-        # Leave message on queue, hoping problem (which probably has to
-        # do with "relaying restrictions") will be fixed shortly
-        defer = 1
-        failure = sys.exc_info()
-        # Try to notify local mailman-owner of the problem by email --
-        # unless the failure is on the mailman-owner address (in which
-        # case the MTA configuration is _seriously_ screwed up)
-        log_hint = ("MTA '%s' relaying restrictions probably "
-                    "too strict" % mm_cfg.SMTPHOST)
-        admin = mm_cfg.MAILMAN_OWNER
-        if sender <> admin:
-            pass
-#             SendTextToUser(log_hint, """\
-# Mailman currently isn't allowed to send mail with sender address
-# 
-#   %(sender)s
-# 
-# In order for your Mailman installation to function properly, sending
-# from this address should be allowed.
-# 
-# The most common cause of such a problem is the MTA's relaying
-# restrictions being set too tight.  You should in your case check the
-# MTA running on '%(mta_host)s'.
-# 
-# As Mailman tries to be MTA independent, you will have to look in your
-# MTA manuals for instructions on how to change it's relaying
-# restriction configuration.""" % {'sender': sender,
-#                                  'mta_host': mm_cfg.SMTPHOST},
-#                            admin, admin)
-        else:
-            log_hint = log_hint + " -- even for sender '%s'" % admin
-    except:
-        # Unanticipated cause of delivery failure - *don't* leave message 
-        # queued, or it may stay, with reattempted delivery, forever...
-        defer = 0
-        failure = sys.exc_info()
-        log_hint = ""
-
-    if defer:
-        OutgoingQueue.deferMessage(queue_entry)        
-    else:
-        OutgoingQueue.dequeueMessage(queue_entry)
-    if failure:
-        t, v = failure[0], failure[1]
-        # XXX Here may be the place to get the failure info back to the list
-        # object, so it can disable the recipient, etc.  But how?
-        l = StampedLogger("smtp-failures", "TrySMTPDelivery", immediate=1)
-        l.write("To %s:\n" % recipient)
-        l.write("\t %s" % t)
-        if v:
-            l.write(' / %s' % v)
-        l.write(' (%s)\n' % (defer and 'deferred' or 'dequeued'))
-        if log_hint:
-            l.write(' %s\n' % log_hint)
-        l.flush()
-
-
 
 def QuotePeriods(text):
     return string.join(string.split(text, '\n.\n'), '\n .\n')
@@ -405,34 +211,46 @@ def ParseAddrs(addresses):
 
 
 
-def GetPathPieces(path):
-    l = string.split(path, '/')
-    try:
-	while 1:
-	    l.remove('')
-    except ValueError:
-	pass
-    return l
+def GetPathPieces(envar='PATH_INFO'):
+    path = os.environ.get(envar)
+    if path:
+        return filter(None, string.split(path, '/'))
+    return None
+
 
 
-nesting_level = None
-def GetNestingLevel():
-  global nesting_level
-  if nesting_level == None:
-    try:
-      path = os.environ['PATH_INFO']
-      if path[0] <> '/': 
-        path= '/' + path
-      nesting_level = len(string.split(path, '/')) - 1
-    except KeyError:
-      nesting_level = 0
-  return nesting_level
+def ScriptURL(target, web_page_url=None, absolute=0):
+    """target - scriptname only, nothing extra
+    web_page_url - the list's configvar of the same name
+    absolute - a flag which if set, generates an absolute url
+    """
+    if web_page_url is None:
+        web_page_url = mm_cfg.DEFAULT_URL
+        if web_page_url[-1] <> '/':
+            web_page_url = web_page_url + '/'
+    fullpath = os.environ.get('REQUEST_URI')
+    if fullpath is None:
+        fullpath = os.environ.get('SCRIPT_NAME', '') + \
+                   os.environ.get('PATH_INFO', '')
+    baseurl = urlparse.urlparse(web_page_url)[2]
+    if not absolute and fullpath[:len(baseurl)] == baseurl:
+        # Use relative addressing
+        fullpath = fullpath[len(baseurl):]
+        i = string.find(fullpath, '?')
+        if i > 0:
+            count = string.count(fullpath, '/', 0, i)
+        else:
+            count = string.count(fullpath, '/')
+        path = ('../' * count) + target
+    else:
+        path = web_page_url + target
+    return path + mm_cfg.CGIEXT
 
 
 
 def MakeDirTree(path, perms=0775, verbose=0):
     made_part = '/'
-    path_parts = GetPathPieces(path)
+    path_parts = filter(None, string.split(path, '/'))
     for item in path_parts:
 	made_part = os.path.join(made_part, item)
 	if os.path.exists(made_part):
@@ -552,6 +370,22 @@ def FindMatchingAddresses(name, dict, *dicts):
 
   
 
+_vowels = ('a', 'e', 'i', 'o', 'u')
+_consonants = ('b', 'c', 'd', 'f', 'g', 'h', 'k', 'm', 'n',
+               'p', 'r', 's', 't', 'v', 'w', 'x', 'z')
+_syllables = []
+
+for v in _vowels:
+    for c in _consonants:
+        _syllables.append(c+v)
+        _syllables.append(v+c)
+
+def MakeRandomPassword(length=6):
+    syls = []
+    while len(syls)*2 < length:
+        syls.append(random.choice(_syllables))
+    return string.join(syls, '')[:length]
+
 def GetRandomSeed():
     chr1 = int(random.random() * 52)
     chr2 = int(random.random() * 52)
@@ -563,22 +397,20 @@ def GetRandomSeed():
         return c
     return "%c%c" % tuple(map(mkletter, (chr1, chr2)))
 
+def SetSiteAdminPassword(pw):
+    fp = open_ex(mm_cfg.SITE_PW_FILE, 'w', perms=0640)
+    fp.write(Crypt.crypt(pw, GetRandomSeed()))
+    fp.close()
 
-def MakeRandomPassword(length=4):
-    password = ""
-    while len(password) < length:
-        password = password + GetRandomSeed()
-    password = password[:length]
-    return password
-
-
-
-def SnarfMessage(msg):
-    if msg.unixfrom:
-	text = msg.unixfrom + string.join(msg.headers, '') + '\n' + msg.body
-    else:
-	text = string.join(msg.headers, '') + '\r\n' + msg.body
-    return (msg.GetSender(), text) 
+def CheckSiteAdminPassword(pw1):
+    try:
+        f = open(mm_cfg.SITE_PW_FILE)
+        pw2 = f.read()
+        f.close()
+        return Crypt.crypt(pw1, pw2[:2]) == pw2
+    # There probably is no site admin password if there was an exception
+    except IOError:
+        return 0
 
 
 
@@ -609,45 +441,13 @@ def ObscureEmail(addr, for_text=0):
     if for_text:
 	return string.replace(addr, "@", " at ")
     else:
-	return string.replace(addr, "@", "__at__")
+	return string.replace(addr, "@", "--at--")
 
 def UnobscureEmail(addr):
     """Invert ObscureEmail() conversion."""
     # Contrived to act as an identity operation on already-unobscured
     # emails, so routines expecting obscured ones will accept both.
-    return string.replace(addr, "__at__", "@")
-
-
-
-def map_maillists(func, names=None, unlock=None, verbose=0):
-    """Apply function (of one argument) to all list objs in turn.
-
-    Returns a list of the results.
-
-    Optional arg 'names' specifies which lists, default all.
-    Optional arg unlock says to unlock immediately after instantiation.
-    Optional arg verbose says to print list name as it's about to be
-    instantiated, CR when instantiation is complete, and result of
-    application as it shows."""
-    from MailList import MailList
-    if names == None:
-        names = list_names()
-    got = []
-    for i in names:
-	if verbose:
-            print i,
-	l = MailList(i)
-	if verbose:
-            print
-	if unlock and l.Locked():
-	    l.Unlock()
-	got.append(apply(func, (l,)))
-	if verbose:
-            print got[-1]
-	if not unlock:
-	    l.Unlock()
-	del l
-    return got
+    return string.replace(addr, "--at--", "@")
 
 
 
@@ -674,11 +474,12 @@ class SafeDict(UserDict):
 
     This is used in maketext so that editing templates is a bit more robust.
     """
-    def __init__(self, d):
+    def __init__(self, d=None):
         # optional initial dictionary is a Python 1.5.2-ism.  Do it this way
         # for portability
         UserDict.__init__(self)
-        self.update(d)
+        if d is not None:
+            self.update(d)
 
     def __getitem__(self, key):
         try:
@@ -690,13 +491,15 @@ class SafeDict(UserDict):
                 return '<Missing key: %s>' % `key`
 
 
-def maketext(templatefile, dict, raw=0):
+def maketext(templatefile, dict=None, raw=0):
     """Make some text from a template file.
 
     Reads the `templatefile', relative to mm_cfg.TEMPLATE_DIR, does string
     substitution by interpolating in the `dict', and if `raw' is false,
     wraps/fills the resulting text by calling wrap().
     """
+    if dict is None:
+        dict = {}
     file = os.path.join(mm_cfg.TEMPLATE_DIR, templatefile)
     fp = open(file)
     template = fp.read()
@@ -708,16 +511,13 @@ def maketext(templatefile, dict, raw=0):
 
 
 
-# given an IncomingMessage object,
-# test for administrivia (eg subscribe, unsubscribe, etc).
-# the test must be a good guess -- messages that return true
-# get sent to the list admin instead of the entire list.
+# given a Message.Message object, test for administrivia (eg subscribe,
+# unsubscribe, etc).  the test must be a good guess -- messages that return
+# true get sent to the list admin instead of the entire list.
 #
 def IsAdministrivia(msg):
     lines = map(string.lower, string.split(msg.body, "\n"))
-    #
     # check to see how many lines that actually have text in them there are
-    # 
     admin_data = {"subscribe": (0, 3),
                   "unsubscribe": (0, 1),
                   "who": (0,0),
@@ -760,24 +560,6 @@ def IsAdministrivia(msg):
     return 0
 
         
-
-def reraise(exc=None, val=None):
-    """Use this function to re-raise an exception.
-
-    This implementation hides the differences between Python versions.
-    Optional exc is the exception type to raise.  When exc is not None,
-    optional val is the exception value to raise.
-
-    """
-    # Python 1.5.2
-    # raise
-    # Earlier Python versions
-    if exc is None:
-        t, v, tb = sys.exc_info()
-        raise t, v, tb
-    raise exc, val, sys.exc_info()[2]
-
-
 
 def mkdir(dir, mode=02775):
     """Wraps os.mkdir() in a umask saving try/finally.
@@ -825,8 +607,9 @@ def open_ex(filename, mode='r', bufsize=-1, perms=0664):
             fp = os.fdopen(fd, mode, bufsize)
             return fp
         # transform any os.errors into IOErrors
-        except os.error, e:
-            reraise(IOError, e)
+        except OSError, e:
+            e.__class__ = IOError
+            raise IOError, e, sys.exc_info()[2]
     finally:
         os.umask(ou)
 
@@ -850,3 +633,68 @@ def GetRequestURI(fallback=None):
         return os.environ['SCRIPT_NAME'] + os.environ['PATH_INFO']
     else:
         return fallback
+
+
+
+# Wait on a dictionary of child pids
+def reap(kids, func=None):
+    while kids:
+        if func:
+            func()
+        pid, status = os.waitpid(-1, os.WNOHANG)
+        if pid <> 0:
+            try:
+                del kids[pid]
+            except KeyError:
+                # Huh?  How can this happen?
+                pass
+
+
+# Useful conversion routines
+# unhexlify(hexlify(s)) == s
+#
+# Python 2.0 has these in the binascii module
+try:
+    from binascii import hexlify, unhexlify
+except ImportError:
+    # Not the most efficient of implementations, but good enough for older
+    # versions of Python.
+    def hexlify(s):
+        acc = []
+        def munge(byte, append=acc.append, a=ord('a'), z=ord('0')):
+            if byte > 9: append(byte+a-10)
+            else: append(byte+z)
+        for c in s:
+            hi, lo = divmod(ord(c), 16)
+            munge(hi)
+            munge(lo)
+        return string.join(map(chr, acc), '')
+
+    def unhexlify(s):
+        acc = []
+        append = acc.append
+        # In Python 2.0, we can use the int() built-in
+        int16 = string.atoi
+        for i in range(0, len(s), 2):
+            append(chr(int16(s[i:i+2], 16)))
+        return string.join(acc, '')
+
+
+
+def write(*args, **kws):
+    file = sys.stdout
+    sep = ' '
+    end = '\n'
+    if kws.has_key('file'):
+        file = kws['file']
+        del kws['file']
+    if kws.has_key('nl'):
+        if not kws['nl']:
+            end = ' '
+        del kws['nl']
+    if kws.has_key('sep'):
+        sep = kws['sep']
+        del kws['sep']
+    if kws:
+        raise TypeError('unexpected keywords')
+    file.write(string.join(map(str, args), sep) + end)
